/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.middleware.wscf.completionHandlers;

import java.rmi.RemoteException;
import java.util.ArrayList;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.oasis.wscaf.core.wscf.exceptions.ResponseFault;
import org.oasis.wscaf.core.wscf.services.RegistrationService;
import org.oasis.wscaf.core.wscf.types.ParticipantGroup;
import org.oasis.wscaf.core.wscf.types.RegistrationContext;
import org.oasis.wscaf.core.wsctx.exceptions.InvalidContext;
import org.oasis.wscaf.core.wsctx.exceptions.NoContext;
import org.oasis.wscaf.core.wsctx.exceptions.UnknownContext;
import org.oasis.wscaf.core.wsctx.services.ContextManager;
import org.oasis.wscaf.core.wsctx.types.CompletionHandler;
import org.oasis.wscaf.core.wsctx.types.Context;
import org.oasis.wscaf.core.wsctx.types.ContextWrapper;
import org.oasis.wscaf.middleware.wslra.compensator.CompensatorServiceStub;
import org.oasis.wscaf.types.wslra.CompensateDocument;
import org.oasis.wscaf.types.wslra.CompleteDocument;
import org.oasis.wscaf.types.wslra.ForgetDocument;
import org.oasis.wscaf.types.wslra.LRAContextType;
import org.oasis.wscaf.types.wslra.LraContextDocument;
import org.oasis.wscaf.utils.AddressingUtils;
import org.oasis.wscaf.utils.Constants;
import org.oasis.wscaf.utils.SOAPUtils;
import org.oasis.wscaf.utils.exceptions.AddressingNotSupported;

public class LRACompletionHandler extends CompletionHandler {
	
	public EndpointReference getCoordinator(ContextWrapper context) throws AxisFault, XmlException, XMLStreamException, AddressingNotSupported {
		ArrayList augmenters = context.getContext().getAugmenters();
		for (int i=0;i<augmenters.size();i++) {
			XmlObject augmenter = (XmlObject) augmenters.get(i);
			if (augmenter.getDomNode().getNamespaceURI().equals(Constants.COORDINATOR_SERVICE.getNamespaceURI())) {
				if (augmenter.getDomNode().getNodeName().equals(Constants.COORDINATOR_SERVICE.getLocalPart())) {
					return AddressingUtils.xBeanToEPR(augmenter);
				}
			}
		}
		return new EndpointReference(AddressingConstants.Final.WSA_ANONYMOUS_URL);
	}
	
    public String complete(ContextWrapper context, String status) {
        System.out.println("[LRACompletionHandler] Completing \"" + context.getContextIdentifier() + "\"");
        boolean success = false;
        boolean fail    = false;
        boolean forget  = false;
        
        if (!(context.getStatus().isRunning())) {
        	return "LRA already completed or completing now";
        }
        
        if (status==null || status.equalsIgnoreCase(Constants.FORGET_NAME)) {
            forget = true;
            context.setStatus(Constants.STATE_LRA_COMPLETING);
        }
        else if (status.equalsIgnoreCase(Constants.SUCCESS_NAME)) {
            success = true;
            context.setStatus(Constants.STATE_LRA_COMPLETING);
        }
        else if (status.equalsIgnoreCase(Constants.FAIL_NAME)) {
            fail = true;
            context.setStatus(Constants.STATE_LRA_COMPENSATING);
        }
        else {
            forget = true;
            context.setStatus(Constants.STATE_LRA_COMPLETING);
        }
        
        // Recursively invoke complete on children
        ContextManager contextManager = ContextManager.getInstance();
        ContextWrapper[] children = contextManager.getChildren(context);
        for (int i=0; i<children.length; i++)
            complete(children[i], status);
        
        
        QName currentStatus = null;
        // Complete the LRA
        if (context.getContext() instanceof RegistrationContext) {
            RegistrationContext registrationContext = (RegistrationContext)context.getContext();
            
            String[] protocols = {"LRA"};
            ParticipantGroup participants = registrationContext.getParticipants(protocols);
            
            // If success and it has a parent, pass its participants to its parent.
            if ((success) && (context.getParent() != null)) {
            	try {
					System.out.println("[LRACompletionHandler] It has parents. Registering its participants.");
					Context parent = contextManager.getContents(context.getParent());
					RegistrationContext parentContext;
					
					if (parent instanceof RegistrationContext) {
						parentContext = (RegistrationContext) parent;
					}
					else {
						parentContext = RegistrationContext.augmentToRegistrationContext(parent);
						ContextManager manager = ContextManager.getInstance();
						manager.setContents(parentContext);
					}
					
					RegistrationService reg = RegistrationService.getInstance();
					
					reg.addParticipant(parentContext.getContextIdentifier(),participants);
					context.setStatus(Constants.STATE_LRA_COMPLETED);
					return "LRA Completion succeed";
				} catch (UnknownContext e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					context.setStatus(Constants.STATE_LRA_FAILED_TO_COMPLETE);
                    return "Unknown context";
				} catch (InvalidContext e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					context.setStatus(Constants.STATE_LRA_FAILED_TO_COMPLETE);
                    return "Invalid context";
				} catch (NoContext e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					context.setStatus(Constants.STATE_LRA_FAILED_TO_COMPLETE);
                    return "No context present";
				} catch (ResponseFault e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					context.setStatus(Constants.STATE_LRA_FAILED_TO_COMPLETE);
                    return "Cannot add participants";
				}
            }
            
            for (int i=participants.getParticipantsSize()-1; i>=0; i--) {
                EndpointReference participantReference = participants.getParticipant(i).getEndPoint();
                EndpointReference participantEPR = participantReference;
                try {
                	EndpointReference coordinatorEPR = getCoordinator(context);
                	System.out
							.println("[LRACompletionHandler][complete]Got coordinator: "
									+ coordinatorEPR.toString());
                    CompensatorServiceStub stub = new CompensatorServiceStub();
                    stub._getServiceClient().setTargetEPR(participantEPR);
                    LraContextDocument lraContextDocument = LraContextDocument.Factory.newInstance();
                    LRAContextType lraContext = SOAPUtils.translateLRAContext(registrationContext);
                    lraContextDocument.setLraContext(lraContext);
                    
                    /*LRAAssertionType assertion = LRAAssertionType.Factory.newInstance();
                    
                    assertion.setParticipantId(participantEPR.getAddress());*/
                    
                    if (success) {
                        System.out.println("[LRACompletionHandler] Sending success to " + participantEPR.toString());
                        CompleteDocument completeDocument = CompleteDocument.Factory.newInstance();
                        completeDocument.addNewComplete().setParticipantId(participantEPR.getAddress());

                        stub.complete(completeDocument,lraContextDocument);
                    }
                    else if (fail) {
                        System.out.println("[LRACompletionHandler] Sending fail to " + participantEPR.toString());
                        CompensateDocument compensateDocument = CompensateDocument.Factory.newInstance();
                        compensateDocument.addNewCompensate().setParticipantId(participantEPR.getAddress());
                        
                        stub.compensate(compensateDocument,lraContextDocument);
                    }
                    else if (forget) {
                        System.out.println("[LRACompletionHandler] Sending forget to " + participantEPR.toString());
                        ForgetDocument forgetDocument = ForgetDocument.Factory.newInstance();
                        forgetDocument.addNewForget().setParticipantId(participantEPR.getAddress());

                        stub.forget(forgetDocument,lraContextDocument);
                    }
                } catch (RemoteException e) {
                	e.printStackTrace();
                    currentStatus = Constants.STATE_LRA_FAILED_TO_COMPLETE;
                    //return "Error invoking stub";
                } catch (Exception e) {
                	e.printStackTrace();
                    currentStatus = Constants.STATE_LRA_FAILED_TO_COMPLETE;
                    //return "Error creating stub";
                }
            }
        }
        
        if (currentStatus == null) {
        	if (success || forget)
        		currentStatus = Constants.STATE_LRA_COMPLETED;
        	else if (fail)
        		currentStatus = Constants.STATE_LRA_COMPENSATED;
        }
        context.setStatus(currentStatus);
        
        System.out.println("[LRACompletionHandler] Request finished correctly for \"" + context.getContextIdentifier() + "\"");
        return "LRA Completion succeed";
    }

    public String getProtocolId() {
        return "LRA";
    }

}
