package org.oasis.wscaf.middleware.wscf.registration;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.llom.factory.OMLinkedListImplFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.addressing.EndpointReferenceHelper;
import org.apache.axis2.context.MessageContext;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.oasis.wscaf.core.wscf.exceptions.ResponseFault;
import org.oasis.wscaf.core.wscf.services.RegistrationService;
import org.oasis.wscaf.core.wscf.types.Participant;
import org.oasis.wscaf.core.wscf.types.ParticipantGroup;
import org.oasis.wscaf.core.wscf.types.ParticipantPair;
import org.oasis.wscaf.core.wscf.types.ParticipantPairGroup;
import org.oasis.wscaf.core.wsctx.types.ContextIdentifier;
import org.oasis.wscaf.types.reference.ServiceRefType;
import org.oasis.wscaf.types.wscf.AddParticipantDocument;
import org.oasis.wscaf.types.wscf.GetParticipantsDocument;
import org.oasis.wscaf.types.wscf.GetStatusDocument;
import org.oasis.wscaf.types.wscf.ParticipantAddedDocument;
import org.oasis.wscaf.types.wscf.ParticipantGroupType;
import org.oasis.wscaf.types.wscf.ParticipantListDocument;
import org.oasis.wscaf.types.wscf.ParticipantPairType;
import org.oasis.wscaf.types.wscf.ParticipantRemovedDocument;
import org.oasis.wscaf.types.wscf.ParticipantReplacedDocument;
import org.oasis.wscaf.types.wscf.RegistrationReplacedDocument;
import org.oasis.wscaf.types.wscf.RemoveParticipantDocument;
import org.oasis.wscaf.types.wscf.ReplaceParticipantDocument;
import org.oasis.wscaf.types.wscf.ReplaceParticipantGroupType;
import org.oasis.wscaf.types.wscf.ResponseFaultType;
import org.oasis.wscaf.types.wscf.StatusDocument;
import org.oasis.wscaf.types.wscf.ParticipantAddedDocument.ParticipantAdded;
import org.oasis.wscaf.types.wscf.ParticipantListDocument.ParticipantList;
import org.oasis.wscaf.types.wscf.ParticipantRemovedDocument.ParticipantRemoved;
import org.oasis.wscaf.types.wscf.ParticipantReplacedDocument.ParticipantReplaced;
import org.oasis.wscaf.types.wscf.ResponseFaultType.FaultDetail;
import org.oasis.wscaf.utils.AddressingUtils;
import org.oasis.wscaf.utils.Constants;
import org.oasis.wscaf.utils.exceptions.AddressingNotSupported;

public class RegistrationServiceSkeleton implements RegistrationServiceSkeletonInterface {

	public ParticipantAddedDocument addParticipant(AddParticipantDocument addParticipant) throws AxisFault {
		System.out.println("[RegistrationService][AddParticipant] Request received");
		
		ContextIdentifier ctxId = new ContextIdentifier((String) MessageContext.getCurrentMessageContext().getProperty(Constants.CurrentContextIdentifier));
		
		RegistrationService registrationService = RegistrationService.getInstance();
		ParticipantGroupType[] participantGroupArray = addParticipant.getAddParticipant().getParticipantsArray();
		ParticipantGroup[] participantGroups = null;
		
		ResponseFault fault = null;
		
		try {
			participantGroups = translateParticipantGroups(participantGroupArray, null);
		} catch (Exception e1) {
			e1.printStackTrace();
			fault = new ResponseFault(0,new AxisFault(e1));
		} 
		// System.out.println("[RegistrationService][AddParticipant] Invoking
		// business logic");
		
		if (fault == null) {
			try {
				registrationService.addParticipant(ctxId, participantGroups);
			} catch (ResponseFault e) {
				e.printStackTrace();
				fault = e;
			}
		}
	
		ParticipantAddedDocument participantAddedDocument = ParticipantAddedDocument.Factory.newInstance();
		ParticipantAdded participantAdded = participantAddedDocument.addNewParticipantAdded();
		if (fault != null) {
			participantAdded.setFault(buidResponseFaultType(fault));
		}
		
		System.out.println("[RegistrationService][AddParticipant] Request end");
	
		return participantAddedDocument;
	}

	public ParticipantListDocument getParticipants(GetParticipantsDocument getParticipants) throws AxisFault {
		System.out.println("[RegistrationService][GetParticipants] Request received");
		
		ContextIdentifier ctxId = new ContextIdentifier((String) MessageContext.getCurrentMessageContext().getProperty(Constants.CurrentContextIdentifier));
		
		RegistrationService registrationService = RegistrationService.getInstance();

		String protocols[] = getParticipants.getGetParticipants().getProtocolArray();
		
		//System.out.println("[RegistrationService][GetParticipants] Invoking business logic");
		ParticipantGroup participants = registrationService.getParticipants(ctxId, protocols);
		
		ParticipantListDocument participantListDocument = ParticipantListDocument.Factory.newInstance();
		ParticipantList participantList = participantListDocument.addNewParticipantList();
		ParticipantGroupType participantGroup = participantList.addNewParticipants();
		
		if (participants == null || participants.getParticipantsSize()==0 ||
			participants.getProtocols() == null || participants.getProtocols().length==0) {
		       	participantList.setNil();
		}
		else {
			String protocolsIncluded[] = participants.getProtocols();
			for (int i=0; i<protocolsIncluded.length; i++) {
				participantGroup.addProtocol(protocolsIncluded[i]);
			}
			ServiceRefType[] participantsRefs = new ServiceRefType[participants.getParticipantsSize()];
			for (int i=0; i<participants.getParticipantsSize(); i++) {
				try {
					participantsRefs[i] = AddressingUtils.eprToServiceRef(participants.getParticipant(i).getEndPoint());
				} catch (XmlException e) {
					e.printStackTrace();
				}
		    }
		    participantGroup.setParticipantArray(participantsRefs);
		}
		
		System.out.println("[RegistrationService][GetParticipants] Ending request");
		
		return participantListDocument;
	}

	public StatusDocument getStatus(GetStatusDocument getStatus) throws AxisFault {
		System.out.println("[RegistrationService][GetStatus] Request received");
		
		ContextIdentifier ctxId = new ContextIdentifier((String) MessageContext.getCurrentMessageContext().getProperty(Constants.CurrentContextIdentifier));
		
		RegistrationService registrationService = RegistrationService.getInstance();

		//System.out.println("[RegistrationService][GetStatus] Invoking business logic");
		QName status = registrationService.getStatus(ctxId);
		
		StatusDocument statusDocument = StatusDocument.Factory.newInstance();
		
		if (status!=null){
			statusDocument.setStatus(status);
		}
		
		System.out.println("[RegistrationService][GetStatus] Ending request");
		
		return statusDocument;
	}

	public void registrationReplaced(RegistrationReplacedDocument registrationReplaced) throws AxisFault {
		System.out.println("[RegistrationService][RegistrationReplaced] Request received");
		
		ContextIdentifier ctxId = new ContextIdentifier((String) MessageContext.getCurrentMessageContext().getProperty(Constants.CurrentContextIdentifier));
		
		EndpointReference replyTo = MessageContext.getCurrentMessageContext().getFrom();
		RegistrationService registrationService = RegistrationService.getInstance();
		
		//System.out.println("[RegistrationService][RegistrationReplaced] Invoking business logic");
		registrationService.registrationReplaced(ctxId, replyTo);
		
		System.out.println("[RegistrationService][RegistrationReplaced] Ending request");
	}

	public ParticipantRemovedDocument removeParticipant(RemoveParticipantDocument removeParticipant) throws AxisFault {
		System.out.println("[RegistrationService][RemoveParticipant] Request received");
		
		ContextIdentifier ctxId = new ContextIdentifier((String) MessageContext.getCurrentMessageContext().getProperty(Constants.CurrentContextIdentifier));
		
		RegistrationService registrationService = RegistrationService.getInstance();

		ParticipantGroupType[] participantGroupArray = removeParticipant.getRemoveParticipant().getParticipantsArray();
		
		ParticipantGroup[] participantGroups = null;
		
		ResponseFault fault = null;
		try {
			participantGroups = translateParticipantGroups(participantGroupArray, null);
		} catch (Exception e1) {
			e1.printStackTrace();
			fault = new ResponseFault(0,new AxisFault(e1));
		} 

		//System.out.println("[RegistrationService][RemoveParticipant] Invoking business logic");
		if (fault == null) {
			try {
				registrationService.removeParticipant(ctxId, participantGroups);
			} catch (ResponseFault e) {
				fault = e;
			}
		}

		ParticipantRemovedDocument participantRemovedDocument = ParticipantRemovedDocument.Factory.newInstance();
		ParticipantRemoved participantRemoved = participantRemovedDocument.addNewParticipantRemoved();
		if (fault != null) {
			participantRemoved.setFault(buidResponseFaultType(fault));
		}
		
		System.out.println("[RegistrationService][RemoveParticipant] Ending request");
		
		return participantRemovedDocument;
	}

	public ParticipantReplacedDocument replaceParticipant(ReplaceParticipantDocument replaceParticipant) throws AxisFault {
		System.out.println("[RegistrationService][ReplaceParticipant] Request received");
		
		ContextIdentifier ctxId = new ContextIdentifier((String) MessageContext.getCurrentMessageContext().getProperty(Constants.CurrentContextIdentifier));
		
		RegistrationService registrationService = RegistrationService.getInstance();

		ReplaceParticipantGroupType replaceParticipantsArray[] = replaceParticipant.getReplaceParticipant().getParticipantsArray();
		ParticipantPairGroup[] participantPairGroups = new ParticipantPairGroup[replaceParticipantsArray.length];
		
		ResponseFault fault = null;
		for (int i=0; i<replaceParticipantsArray.length; i++) {
		    ParticipantPairGroup participantPairs = new ParticipantPairGroup();
		    ReplaceParticipantGroupType participantPairGroup = replaceParticipantsArray[i];
		    
		    String[] protocolArray = participantPairGroup.getProtocolArray();
		    for (int j=0; j<protocolArray.length; j++)
		        participantPairs.addSubProtocol(protocolArray[j]);
		    
		    ParticipantPairType[] participantPairArray = participantPairGroup.getParticipantPairArray();
		    for (int j=0; j<participantPairArray.length; j++) {
		        ParticipantPairType participantPair = participantPairArray[j];
		        Participant oldParticipant = null;
				Participant newParticipant = null;
				try {
					oldParticipant = new Participant(AddressingUtils.serviceRefToEPR(participantPair.getOldParticipant()));
					newParticipant = new Participant(AddressingUtils.serviceRefToEPR(participantPair.getNewParticipant()));
					participantPairs.addParticipantPair(new ParticipantPair(oldParticipant, newParticipant));
				} catch (AddressingNotSupported e) {
					e.printStackTrace();
					fault = new ResponseFault(0,new AxisFault(e));
				}
		    }
		    
		    participantPairGroups[i] = participantPairs;
		}

		if (fault == null) {
			//System.out.println("[RegistrationService][ReplaceParticipant] Invoking business logic");
			try {
				registrationService.replaceParticipant(ctxId, participantPairGroups);
			} catch (ResponseFault e) {
				fault = e;
			}
		}

		ParticipantReplacedDocument participantReplacedDocument = ParticipantReplacedDocument.Factory.newInstance();
		ParticipantReplaced participantReplaced = participantReplacedDocument.addNewParticipantReplaced();
		
		if (fault != null) {
			participantReplaced.setFault(buidResponseFaultType(fault));
		}
		else {
		    QName status = registrationService.getStatus(ctxId);
		    
		    XmlCursor cursor = participantReplaced.newCursor();
		    cursor.toFirstContentToken();
		    cursor.beginElement(new QName(Constants.COORDINATION_NS, "status"));
		    cursor.insertChars(status.toString());
		}
		
		System.out.println("[RegistrationService][ReplaceParticipant] Ending request");
		
		return participantReplacedDocument;
	}
	
	private ParticipantGroup[] translateParticipantGroups(ParticipantGroupType[] participantGroupArray, EndpointReference epr) throws AxisFault, XmlException, XMLStreamException, AddressingNotSupported {
        //EndpointReference creator = null;
        if (epr!=null) {
            //creator = (EndpointReference) AddressingUtils.cloneEPR(epr);
            ServiceRefType element = null;
			
            OMElement referenceOM = EndpointReferenceHelper.toOM(new OMLinkedListImplFactory(), epr, new QName(Constants.CONTEXT_NS,"creator","ctx"), AddressingConstants.Final.WSA_NAMESPACE);
			element = ServiceRefType.Factory.parse(referenceOM.toString());
            element.setReferenceScheme(AddressingConstants.Final.WSA_NAMESPACE);
        }
        ParticipantGroup[] participantGroups = new ParticipantGroup[participantGroupArray.length];
        for (int i=0; i<participantGroupArray.length; i++) {
            ParticipantGroup participants = new ParticipantGroup();    
            ParticipantGroupType participantGroup = participantGroupArray[i];
            
            System.out
					.println("[WSCAFRegistrationServiceSkeleton][translateParticipantGroups] Inserting participants for: "
							+ participantGroup.toString());
            
            String[] protocolArray = participantGroup.getProtocolArray();
            for (int j=0; j<protocolArray.length; j++) {
            	System.out
						.println("[WSCAFRegistrationServiceSkeleton][translateParticipantGroups] Inserting elements of protocol: "
								+ protocolArray[j]);
                         participants.addProtocol(protocolArray[j]);
            }
                
            
            ServiceRefType[] participantArray = participantGroup.getParticipantArray();
            for (int j=0; j<participantArray.length; j++) {
                Participant participant = new Participant(AddressingUtils.serviceRefToEPR(/*participantArray[j]*/participantGroup.getParticipantArray(j)));
                participants.addParticipant(participant);
            }
            participantGroups[i]=participants;
        }
        return participantGroups;
    }
    
    private ResponseFaultType buidResponseFaultType(ResponseFault fault) {
    	
        ResponseFaultType responseFault = ResponseFaultType.Factory.newInstance();
        if (fault==null)
            responseFault.setNil();
        else {
            responseFault.setSuccessCount(fault.getSuccessCount());
            responseFault.setFaultCode(fault.getFaultCode());
            FaultDetail faultDetail = responseFault.addNewFaultDetail();
            
            if (fault.getFaultDetailElement() != null) {
            	XmlObject faultObject = null;
            	try {
            		faultObject = XmlObject.Factory.parse(fault.getFaultDetailElement().getXMLStreamReader());
            	} catch (XmlException e) {
            		// TODO Auto-generated catch block
            		e.printStackTrace();
            	}
            	if (faultObject != null) {
            		faultDetail.set(faultObject);
            
            		//SOAPUtils.translateFault(fault.getFaultDetail(), faultDetail.newCursor());
            	}
            }
        }
        return responseFault;
    }

}
