package org.oasis.wscaf.middleware.wsctx.manager;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.xmlbeans.XmlException;
import org.oasis.wscaf.core.wsctx.exceptions.InvalidContextStructure;
import org.oasis.wscaf.core.wsctx.services.ContextManager;
import org.oasis.wscaf.core.wsctx.types.Context;
import org.oasis.wscaf.core.wsctx.types.ContextIdentifier;
import org.oasis.wscaf.types.wsctx.ContentsDocument;
import org.oasis.wscaf.types.wsctx.ContentsSetDocument;
import org.oasis.wscaf.types.wsctx.ContextType;
import org.oasis.wscaf.types.wsctx.GetContentsDocument;
import org.oasis.wscaf.types.wsctx.SetContentsDocument;
import org.oasis.wscaf.types.wsctx.ContentsDocument.Contents;
import org.oasis.wscaf.utils.Constants;
import org.oasis.wscaf.utils.SOAPUtils;

public class ContextManagerSkeleton implements ContextManagerSkeletonInterface {

	public ContentsDocument getContents(GetContentsDocument getContents) throws AxisFault {
		System.out.println("[ContextManager][GetContents] Request received");
		
		ContextIdentifier ctxId = new ContextIdentifier((String) MessageContext.getCurrentMessageContext().getProperty(Constants.CurrentContextIdentifier));
				
		ContextManager contextManager = ContextManager.getInstance();

		//System.out.println("[ContextManager][GetContents] Invoking business logic");
		Context context = contextManager.getContents(ctxId);
		    
		ContentsDocument contentsDocument = ContentsDocument.Factory.newInstance();
		Contents contents = contentsDocument.addNewContents();
		
		try {
			contents.setContext(SOAPUtils.translateContext(context));
		} catch (XmlException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		    
		System.out.println("Contents Document:\n" + contentsDocument.toString());
		
		System.out.println("[ContextManager][GetContents] Ending request");
		return contentsDocument;
	}
	

	public ContentsSetDocument setContents(SetContentsDocument setContents) throws AxisFault {
		
		System.out.println("[ContextManager][SetContents] Request received");
		
		ContextIdentifier ctxId = new ContextIdentifier((String) MessageContext.getCurrentMessageContext().getProperty(Constants.CurrentContextIdentifier));
		
		ContextManager contextManager = ContextManager.getInstance();

		ContextType newContext = setContents.getSetContents().getContext();
		newContext.getContextIdentifier().setStringValue(ctxId.toString());
		
		//System.out.println("[ContextManager][SetContents] Invoking business logic");
		try {
			contextManager.setContents(SOAPUtils.translateContext(newContext, null));
		} catch (Exception e) {
			e.printStackTrace();
			throw new InvalidContextStructure("The incomming contents is not a valid context: "+e.getMessage());
		}
		
		ContentsSetDocument contentsSetDocument = ContentsSetDocument.Factory.newInstance();
		contentsSetDocument.addNewContentsSet();
		
		System.out.println("[ContextManager][SetContents] Ending request");
		
		return contentsSetDocument;
	}

}
