package org.oasis.wscaf.tests;

import java.rmi.RemoteException;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.OperationContext;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.xmlbeans.XmlException;
import org.oasis.wscaf.middleware.wsctx.manager.ContextManagerStub;
import org.oasis.wscaf.middleware.wsctx.service.ContextServiceStub;
import org.oasis.wscaf.types.wsctx.BeginDocument;
import org.oasis.wscaf.types.wsctx.ContextDocument;
import org.oasis.wscaf.types.wsctx.ContextType;
import org.oasis.wscaf.types.wsctx.GetContentsDocument;
import org.oasis.wscaf.types.wsctx.SetContentsDocument;
import org.oasis.wscaf.utils.AddressingUtils;
import org.oasis.wscaf.utils.Constants;
import org.oasis.wscaf.utils.ContextUtils;
import org.oasis.wscaf.utils.exceptions.ElementNotFound;

public class ContextManagerTest {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			EndpointReference ctxServiceRef = new EndpointReference("http://localhost:8081/axis2/services/ContextService");
			EndpointReference ctxManagerRef = new EndpointReference("http://localhost:8081/axis2/services/ContextManager");
			
			ContextServiceStub stubSrvc = new ContextServiceStub(ctxServiceRef.getAddress());
			stubSrvc._getServiceClient().setCachingOperationContext(true);
			
			//Begin first context
			BeginDocument beginDocument = BeginDocument.Factory.newInstance();
			beginDocument.addNewBegin();
			stubSrvc.begin(beginDocument);
			
			String newId = null;
			OperationContext opContext = stubSrvc._getServiceClient().getLastOperationContext();
			if (opContext != null) {
				MessageContext messageContext = opContext.getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
				System.out.println("IN Message contents: \n"
						+ messageContext.getEnvelope().getHeader());
				ContextUtils.getContext(messageContext, Constants.CONTEXT_NAME);
				newId = (String) messageContext.getProperty(Constants.CurrentContextIdentifier);
			}
			else {
				throw new Exception("Cannot get operation context");
			}
			
			//Set contents
			ContextDocument ctx = ContextDocument.Factory.newInstance();
			
			ctx.addNewContext().addNewContextIdentifier().setStringValue(newId);
			
			ContextManagerStub stubMngr = new ContextManagerStub(ctxManagerRef.getAddress());
			
			SetContentsDocument setContents = SetContentsDocument.Factory.newInstance();
			ContextType contextContents = setContents.addNewSetContents().addNewContext();
			contextContents.addNewContextIdentifier().setStringValue(newId);
			contextContents.setContextManager(AddressingUtils.eprToServiceRef(ctxManagerRef));
			contextContents.setContextService(AddressingUtils.eprToServiceRef(ctxServiceRef));
			
			stubMngr.setContents(setContents,ctx);
			
			//Get contents
			GetContentsDocument getContents = GetContentsDocument.Factory.newInstance();
			getContents.addNewGetContents();
			
			stubMngr.getContents(getContents, ctx);
			
		} catch (AxisFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XmlException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ElementNotFound e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}


	}

}
