package org.oasis.wscaf.tests;

import java.rmi.RemoteException;

import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.OperationContext;
import org.apache.axis2.wsdl.WSDLConstants;
import org.oasis.wscaf.middleware.wsctx.service.ContextServiceStub;
import org.oasis.wscaf.types.wsctx.BeginDocument;
import org.oasis.wscaf.types.wsctx.CompleteDocument;
import org.oasis.wscaf.types.wsctx.ContextDocument;
import org.oasis.wscaf.types.wsctx.ContextType;
import org.oasis.wscaf.types.wsctx.GetStatusDocument;
import org.oasis.wscaf.utils.Constants;
import org.oasis.wscaf.utils.ContextUtils;
import org.oasis.wscaf.utils.ConversionUtils;
import org.oasis.wscaf.utils.exceptions.ElementNotFound;

public class ContextServiceTest {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			
			EndpointReference ctxServiceRef = new EndpointReference("http://localhost:8081/axis2/services/ContextService");
			
			ContextServiceStub stub = new ContextServiceStub(ctxServiceRef.getAddress());
			stub._getServiceClient().setCachingOperationContext(true);
			
			//Begin first context
			BeginDocument beginDocument = BeginDocument.Factory.newInstance();
			beginDocument.addNewBegin();
			stub.begin(beginDocument);
			
			String newId = null;
			OperationContext opContext = stub._getServiceClient().getLastOperationContext();
			if (opContext != null) {
				MessageContext messageContext = opContext.getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
				System.out.println("IN Message contents: \n"
						+ messageContext.getEnvelope().getHeader());
				ContextUtils.getContext(messageContext, Constants.CONTEXT_NAME);
				newId = (String) messageContext.getProperty(Constants.CurrentContextIdentifier);
			}
			else {
				throw new Exception("Cannot get operation context");
			}
			//Begin nested context
			ContextDocument ctx = ContextDocument.Factory.newInstance();
			ContextType context = ctx.addNewContext();
			context.addNewContextIdentifier().setStringValue(newId);
			
			stub._getServiceClient().addHeader(ConversionUtils.xBeanToOM(ctx));
			
			stub.begin(beginDocument);
			
			stub._getServiceClient().removeHeaders();
			
			//Get status before completion
			GetStatusDocument getStatusDocument = GetStatusDocument.Factory.newInstance();
			getStatusDocument.addNewGetStatus();
			
			stub.getStatus(getStatusDocument, ctx);
			
			//Complete context
			CompleteDocument completeDocument = CompleteDocument.Factory.newInstance();
			completeDocument.addNewComplete();
			
			stub.complete(completeDocument, ctx);
			
			//Get status after completion
			stub.getStatus(getStatusDocument, ctx);
			
		} catch (AxisFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XMLStreamException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
	}

}
