/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.utils;

import java.io.IOException;
import java.util.Iterator;

import javax.xml.namespace.QName;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.llom.factory.OMLinkedListImplFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.addressing.EndpointReferenceHelper;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.custommonkey.xmlunit.Diff;
import org.custommonkey.xmlunit.XMLUnit;
import org.oasis.wscaf.types.reference.ServiceRefType;
import org.oasis.wscaf.utils.exceptions.AddressingNotSupported;
import org.xml.sax.SAXException;

public class AddressingUtils {

	public static EndpointReference xBeanToEPR(XmlObject ref) throws AddressingNotSupported {
        ServiceRefType serviceRef;
		try {
			serviceRef = ServiceRefType.Factory.parse(ref.getDomNode());
		} catch (XmlException e) {
			e.printStackTrace();
			throw new AddressingNotSupported(e);
		}        
        return serviceRefToEPR(serviceRef);
    }
	
	public static EndpointReference serviceRefToEPR(ServiceRefType serviceRef) throws AddressingNotSupported {
        if (serviceRef==null)
            return null;
        
        if (serviceRef.isNil())
        	return null;
        
        //System.out.println("[AddressugUtils][serviceRefToEPR]: **Getting epr from: "+ serviceRef.toString());
        
        String scheme = serviceRef.getReferenceScheme();
        if (scheme != null) {
        	if (!scheme.equals(AddressingConstants.Final.WSA_NAMESPACE)) {
        		throw new AddressingNotSupported(scheme+" not supported. Only "+AddressingConstants.Final.WSA_NAMESPACE+" is supported.");
        	}
        }
        
        QName eprName = Constants.ENDPOINTREFERENCE_FINAL_NAME;
        
        try {
			OMElement xbeanOM = ConversionUtils.xBeanToOM(serviceRef);
			Iterator eprs = xbeanOM.getChildrenWithName(eprName);
			
			
			
			if (eprs.hasNext()) {
				OMElement eprOM = (OMElement) eprs.next();
				//System.out.println("[AddressugUtils][serviceRefToEPR]: **Found EndpointReference in: "+ eprOM);
				return EndpointReferenceHelper.fromOM(eprOM);
			}
			else {
				throw new AddressingNotSupported("No "+eprName.toString()+" element found in service ref "+serviceRef.toString());
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new AddressingNotSupported(e);
		} 
    }
    
    public static OMElement eprToOM(EndpointReference source) throws AxisFault {
    	return eprToOM(source,Constants.ENDPOINTREFERENCE_FINAL_NAME);
    }
    
    private static OMElement eprToOM(EndpointReference source, QName name) throws AxisFault {
    	return EndpointReferenceHelper.toOM(new OMLinkedListImplFactory(), source, name, AddressingConstants.Final.WSA_NAMESPACE);
    }
    
    public static ServiceRefType eprToServiceRef(EndpointReference ref) throws AxisFault, XmlException {
    	//System.out.println("[AddressugUtils][serviceRefToEPR]: **Generating serviceRef for: "+ ref.toString());
    	
    	OMElement element = eprToOM(ref);
    	
    	//System.out.println("[AddressugUtils][serviceRefToEPR]: Generated omelement: "+ element.toString());
    	
    	ServiceRefType resRef = ServiceRefType.Factory.newInstance();
    	XmlCursor cursor = resRef.newCursor();
    	cursor.toFirstContentToken();
    	XmlObject eprBean = ConversionUtils.omToXBean(element);
    	XmlCursor objectCursor = eprBean.newCursor();
    	objectCursor.toFirstContentToken();
    	objectCursor.copyXml(cursor);
    	
    	resRef.setReferenceScheme(AddressingConstants.Final.WSA_NAMESPACE);
    	//System.out.println("[AddressugUtils][eprToServiceRef]: **Generated service ref: "+ resRef.toString());
    	return resRef;
    }
    
    public static boolean compareEPR(EndpointReference epr1, EndpointReference epr2) throws SAXException, IOException, ParserConfigurationException {
    	//QName elementName = new QName(AddressingConstants.Final.WSA_NAMESPACE,"EndpointReference");
    	OMElement elemEpr1 = eprToOM(epr1);
    	OMElement elemEpr2 = eprToOM(epr2);
    	Diff diff = XMLUnit.compareXML(elemEpr1.toString(), elemEpr2.toString());
    	return diff.similar();
    	//XMLComparator comparator = new XMLComparator();
    	//return comparator.compare(elemEpr1, elemEpr2);
    	
    }
}
