/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.utils;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Stub;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.oasis.wscaf.types.reference.ServiceRefType;
import org.oasis.wscaf.types.wscf.AddParticipantDocument;
import org.oasis.wscaf.types.wscf.GetParticipantsDocument;
import org.oasis.wscaf.types.wscf.ParticipantGroupType;
import org.oasis.wscaf.types.wscf.ParticipantPairType;
import org.oasis.wscaf.types.wscf.RegistrationContextDocument;
import org.oasis.wscaf.types.wscf.RegistrationContextType;
import org.oasis.wscaf.types.wscf.RemoveParticipantDocument;
import org.oasis.wscaf.types.wscf.ReplaceParticipantDocument;
import org.oasis.wscaf.types.wscf.ReplaceParticipantGroupType;
import org.oasis.wscaf.types.wscf.AddParticipantDocument.AddParticipant;
import org.oasis.wscaf.types.wscf.GetParticipantsDocument.GetParticipants;
import org.oasis.wscaf.types.wscf.GetStatusDocument.GetStatus;
import org.oasis.wscaf.types.wscf.RemoveParticipantDocument.RemoveParticipant;
import org.oasis.wscaf.types.wscf.ReplaceParticipantDocument.ReplaceParticipant;
import org.oasis.wscaf.types.wsctx.AssertionType;
import org.oasis.wscaf.types.wsctx.BeginDocument;
import org.oasis.wscaf.types.wsctx.CompleteDocument;
import org.oasis.wscaf.types.wsctx.ContextDocument;
import org.oasis.wscaf.types.wsctx.ContextIdentifierType;
import org.oasis.wscaf.types.wsctx.ContextType;
import org.oasis.wscaf.types.wsctx.GetContentsDocument;
import org.oasis.wscaf.types.wscf.GetStatusDocument;
import org.oasis.wscaf.types.wsctx.SetContentsDocument;

public class ClientUtils {

	public static ContextDocument createContext(String id, EndpointReference serviceManager) throws AxisFault, XmlException {
    	ContextDocument registrationContextDocument = ContextDocument.Factory.newInstance();
        ContextType context = registrationContextDocument.addNewContext();
        context.addNewContextIdentifier().setStringValue(id);
        context.setContextManager(AddressingUtils.eprToServiceRef(serviceManager));
        return registrationContextDocument;
    }
	
	public static void addServiceReference(ContextDocument context, QName name, EndpointReference ref) throws AxisFault, XmlException {
		addContextElement(context, name, AddressingUtils.eprToServiceRef(ref));
	}
	
	public static void addContextElement(ContextDocument context, QName name, XmlObject content) {
		ContextType contextContent = context.getContext();
		XmlCursor cursor = contextContent.newCursor();
		cursor.toEndToken();
		cursor.beginElement(name);
		
		XmlCursor objCursor = content.newCursor();
		objCursor.toFirstContentToken();
		objCursor.copyXml(cursor);
	}
	
	public static RegistrationContextDocument createRegistrationContext(String id, EndpointReference serviceManager, EndpointReference registrationService) throws AxisFault, XmlException {
		RegistrationContextDocument registrationContextDocument = RegistrationContextDocument.Factory.newInstance();
		RegistrationContextType context = registrationContextDocument.addNewRegistrationContext();
		context.addNewContextIdentifier().setId(id);
		context.setContextManager(AddressingUtils.eprToServiceRef(serviceManager));
		context.setRegistrationService(AddressingUtils.eprToServiceRef(registrationService));
		
		return registrationContextDocument;
	}

    public static BeginDocument createBegin(QName type) {
    	BeginDocument beginDocument = BeginDocument.Factory.newInstance();
        beginDocument.addNewBegin().setType(type);
        return beginDocument;
    }
    
    public static SetContentsDocument createSetContents(ContextIdentifierType id, EndpointReference eprReg, EndpointReference eprCoord) throws AxisFault, XmlException {
    	SetContentsDocument setContentsDocument = SetContentsDocument.Factory.newInstance();
        ContextType newContext = setContentsDocument.addNewSetContents().addNewContext();
        newContext.setContextIdentifier(id);
       
        /*OMElement element = AddressingUtils.eprToOM(eprReg,new QName(COORDINATION_NS,"registration-service"));
     	ServiceRefType resRef = ServiceRefType.Factory.parse(element.toString());
     	resRef.setReferenceScheme(AddressingConstants.Final.WSA_NAMESPACE);*/
     	
     	
     	ServiceRefType resRef = AddressingUtils.eprToServiceRef(eprReg);
     	
        XmlCursor cursor = newContext.newCursor();
        //cursor.toFirstContentToken();
        cursor.toEndToken();
        cursor.push();
        cursor.beginElement(new QName(Constants.COORDINATION_NS,"registration-service"));
        
        XmlCursor refCursor = resRef.newCursor();
     	refCursor.toFirstContentToken();
     	
     	refCursor.copyXml(cursor);
     	
     	/*OMElement coordElement = AddressingUtils.eprToOM(eprCoord,Constants.COORDINATOR_SERVICE);
     	ServiceRefType coordRef = ServiceRefType.Factory.parse(coordElement.toString());
     	coordRef.setReferenceScheme(AddressingConstants.Final.WSA_NAMESPACE);*/
     	
     	ServiceRefType coordRef = AddressingUtils.eprToServiceRef(eprCoord);
     	
     	cursor.pop();
     	
     	XmlCursor coordCursor = coordRef.newCursor();
     	coordCursor.toFirstContentToken();
     	cursor.beginElement(Constants.COORDINATOR_SERVICE);
     	
     	coordCursor.copyXml(cursor);
     	
     	cursor.toStartDoc();
     	//System.out.println("Cursor content: "+cursor.xmlText());
     	//System.out.println("SetContentsDocument contents: "+ setContentsDocument.toString());
     	
     	
        //cursor.beginElement(Constants.PARTICIPANT_NUMBER_NAME);
        //cursor.insertChars("1");
        return setContentsDocument;
    }
    
    public static class ParticipantsType {
    	public String protocol;
    	public EndpointReference[] participants;
    	
		public ParticipantsType(String protocol, EndpointReference[] participants) {
			this.protocol = protocol;
			this.participants = participants;
		}
    	
    }
    
    public static ParticipantGroupType[] createParticipantGroupTypeArray(ParticipantsType[] participants) throws AxisFault, XmlException {
    	ParticipantGroupType[] res = new ParticipantGroupType[participants.length];
    	for (int i=0;i<participants.length;i++) {
        	res[i] = ParticipantGroupType.Factory.newInstance();
        	res[i].addProtocol(participants[i].protocol);
        
        	
        	ServiceRefType[] participants1 = new ServiceRefType[participants[i].participants.length];
        	for (int j=0;j<participants1.length;j++) {
        		participants1[j] = AddressingUtils.eprToServiceRef(participants[i].participants[j]);
        	}
        	//ServiceRefType participant1 = participantGroup1.addNewParticipant();
        	//participants1[0] = AddressingUtils.eprToServiceRef(new EndpointReference("address1"));
        	//SOAPUtils.writeEPR(participant1, "address1", "portType1");
        	res[i].setParticipantArray(participants1);
        }
    	return res;
    }
    
    public static AddParticipantDocument createAddParticipant(ParticipantsType[] participants) throws AxisFault, XmlException {
    	AddParticipantDocument addParticipantDocument = AddParticipantDocument.Factory.newInstance();
        AddParticipant addParticipant = AddParticipant.Factory.newInstance();
        ParticipantGroupType[] participantsArray = createParticipantGroupTypeArray(participants);
        addParticipant.setParticipantsArray(participantsArray);
        addParticipantDocument.setAddParticipant(addParticipant);
        return addParticipantDocument;
    }
    
    public static GetParticipantsDocument createGetParticipants(String protocol) {
    	GetParticipantsDocument getParticipantsDocument = GetParticipantsDocument.Factory.newInstance();
        GetParticipants getParticipants = GetParticipants.Factory.newInstance();
        getParticipants.addProtocol(protocol);
        getParticipantsDocument.setGetParticipants(getParticipants);
        return getParticipantsDocument;
    }
    
    public static RemoveParticipantDocument createRemoveParticipant(ParticipantsType[] participants) throws AxisFault, XmlException{
    	RemoveParticipantDocument removeParticipantDocument = RemoveParticipantDocument.Factory.newInstance();
        RemoveParticipant removeParticipant = RemoveParticipant.Factory.newInstance();
        ParticipantGroupType[] participantsArray = createParticipantGroupTypeArray(participants);
        removeParticipant.setParticipantsArray(participantsArray);
        removeParticipantDocument.setRemoveParticipant(removeParticipant);
        return removeParticipantDocument;
    }
    
    public static class ReplaceParticipantType {
    	public String protocol;
    	public EndpointReference oldParticipant;
    	public EndpointReference newParticipant;
    	
		public ReplaceParticipantType(String protocol, EndpointReference oldParticipant, EndpointReference newParticipant) {
			this.protocol = protocol;
			this.oldParticipant = oldParticipant;
			this.newParticipant = newParticipant;
		}
    }
    
    public static ReplaceParticipantGroupType[] createReplaceParticipantGroupTypes(ReplaceParticipantType[] participants) throws AxisFault, XmlException {
    	ReplaceParticipantGroupType[] res = new ReplaceParticipantGroupType[participants.length];
    	for (int i=0;i<participants.length;i++) {
        	res[i] = ReplaceParticipantGroupType.Factory.newInstance();
        	res[i].addProtocol(participants[i].protocol);
        
        	ServiceRefType oldParticipant = AddressingUtils.eprToServiceRef(participants[i].oldParticipant);
        	ServiceRefType newParticipant = AddressingUtils.eprToServiceRef(participants[i].newParticipant);
        	
        	ParticipantPairType participantPair = res[i].addNewParticipantPair();
        	participantPair.setOldParticipant(oldParticipant);
        	participantPair.setNewParticipant(newParticipant);
        }
    	return res;
    }
    
    public static ReplaceParticipantDocument createReplaceParticipant(ReplaceParticipantType[] replaces) throws AxisFault, XmlException {
    	ReplaceParticipantDocument replaceParticipantDocument = ReplaceParticipantDocument.Factory.newInstance();
        ReplaceParticipant replaceParticipant = ReplaceParticipant.Factory.newInstance();
        ReplaceParticipantGroupType[] replacesXBean = createReplaceParticipantGroupTypes(replaces);
        replaceParticipant.setParticipantsArray(replacesXBean);
        replaceParticipantDocument.setReplaceParticipant(replaceParticipant);
        return replaceParticipantDocument;
    }
    
    public static GetStatusDocument createGetStatusDocument() {
    	GetStatusDocument getStatusDocument = GetStatusDocument.Factory.newInstance();
        GetStatus getStatus = GetStatus.Factory.newInstance();
        getStatusDocument.setGetStatus(getStatus);
        return getStatusDocument;
    }
    
    public static CompleteDocument createCompleteDocument(String status) {
    	CompleteDocument completeDocument = CompleteDocument.Factory.newInstance();
        AssertionType complete = AssertionType.Factory.newInstance();
        /** ANY **/
        //System.out.println(newContext);
        XmlCursor cursor = complete.newCursor();
        cursor.toFirstContentToken();
        cursor.beginElement(new QName(Constants.COORDINATION_NS, "status"));
        cursor.insertChars(status);
        //System.out.println(newContext);
        /*********/
        completeDocument.setComplete(complete);
        return completeDocument;
    }
    
    public static GetContentsDocument createGetContentsDocument() {
    	GetContentsDocument getContentsDocument = GetContentsDocument.Factory.newInstance();
        AssertionType getContents = AssertionType.Factory.newInstance();
        getContentsDocument.setGetContents(getContents);
        return getContentsDocument;
    }
    
    public static void putContext(Stub stub, OMElement context) {
    	stub._getServiceClient().removeHeaders();
    	stub._getServiceClient().addHeader(context);
    }
    
    public static void putReply(Stub stub, EndpointReference epr) {
    	stub._getServiceClient().getOptions().setReplyTo(epr);
    }
	
}
