/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.utils;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.soap.SOAPConstants;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.addressing.EndpointReferenceHelper;
import org.apache.axis2.context.MessageContext;
import org.oasis.wscaf.utils.exceptions.AddressingNotSupported;
import org.oasis.wscaf.utils.exceptions.ElementNotFound;

public class ContextUtils {
	
	public static OMElement getContext(MessageContext msgContext, QName contextName) throws ElementNotFound {
		SOAPHeader header = msgContext.getEnvelope().getHeader();
		if (header != null) {
			OMElement context = getContext(header,contextName);
			if (context != null) {
				String contextId = getTextElement(context, Constants.CONTEXT_IDENTIFIER_NAME);
				if (contextId != null) {
					msgContext.setProperty(Constants.CurrentContextProperty, context);
					msgContext.setProperty(Constants.CurrentContextIdentifier, contextId);
					return context;
				}
				else {
					throw new ElementNotFound("Element "+Constants.CONTEXT_IDENTIFIER_NAME+" is not well formed.",Constants.CONTEXT_IDENTIFIER_NAME);
				}
			}
			else {
				throw new ElementNotFound("Context "+contextName+" was not found in header.",contextName);
			}
		}
		else {
			throw new ElementNotFound("Could not find a header in the provided context.",contextName);
		}
	}
	
	public static OMElement getContext(SOAPHeader header, QName contextName) {
		if (header != null) {
			return header.getFirstChildWithName(contextName);
		}
		else {
			return null;
		}
	}

	public static OMElement getContextElement(OMElement context, QName elem_name) throws ElementNotFound {
		OMElement elem = context.getFirstChildWithName(elem_name);
		if (elem != null) {
			return elem;
		}
		else {
			throw new ElementNotFound("Could not find element "+elem_name+" in context",elem_name);
		}
	}
	
	public static EndpointReference getEndpointElement(OMElement context, QName service) throws AddressingNotSupported,ElementNotFound {
		OMElement omService = getContextElement(context, service);
		OMElement ref = (OMElement) omService.getFirstElement();
		if (service !=null) {
			try {
				return EndpointReferenceHelper.fromOM(ref);
			} catch (AxisFault e) {
				e.printStackTrace();
				throw new AddressingNotSupported(e);
			}
		}
		else {
			throw new AddressingNotSupported();
		}
	}
	
	public static String getTextElement(OMElement context, QName name) throws ElementNotFound {
		OMElement omElement = getContextElement(context, name);
		return omElement.getText();
	}
	
	public static void addElement(OMElement context, QName elementName, OMElement content) {
		OMFactory factory = context.getOMFactory();
		OMElement element = factory.createOMElement(elementName, context);
		element.addChild(content);
	}
	
	public static void addService(OMElement context, QName serviceName, EndpointReference ref) throws AxisFault {
		addElement(context,serviceName,AddressingUtils.eprToOM(ref));
	}
	
}
